"""Base class for backend generators."""

from __future__ import annotations

import shutil
from abc import abstractmethod
from pathlib import Path
from typing import TYPE_CHECKING

from litestar_start.core.base import BaseGenerator
from litestar_start.core.utils import ensure_dir, write_file

if TYPE_CHECKING:
    from litestar_start.core.base import GeneratorContext


class BackendGenerator(BaseGenerator):
    """Abstract base class for backend generators."""

    def generate(self, context: GeneratorContext) -> None:
        """Generate backend code.

        Args:
            context: Generator context.

        """
        backend_dir = context.output_dir
        ensure_dir(backend_dir)

        # Generate pyproject.toml
        self._generate_pyproject(backend_dir, context)

        # Generate app structure
        self._generate_app_structure(backend_dir, context)

        # Copy template files (Dockerfile, README, etc.)
        self._copy_template_files(backend_dir, context)

        # Apply plugins
        self._apply_plugins(backend_dir, context)

    @abstractmethod
    def _generate_pyproject(self, backend_dir: Path, context: GeneratorContext) -> None:
        """Generate pyproject.toml for the backend.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        ...

    @abstractmethod
    def _generate_app_structure(self, backend_dir: Path, context: GeneratorContext) -> None:
        """Generate the application structure.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        ...

    def _apply_plugins(self, backend_dir: Path, context: GeneratorContext) -> None:
        """Apply plugins to enhance the backend.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        # Plugin application will be implemented when plugins are created

    @staticmethod
    def _copy_template_files(backend_dir: Path, context: GeneratorContext) -> None:
        """Copy template files to backend directory.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        templates_dir = Path(__file__).parent / "templates"

        # Copy Dockerfile
        if (templates_dir / "Dockerfile").exists():
            shutil.copy2(templates_dir / "Dockerfile", backend_dir / "Dockerfile")

        # Copy .dockerignore
        if (templates_dir / ".dockerignore").exists():
            shutil.copy2(templates_dir / ".dockerignore", backend_dir / ".dockerignore")

        # Copy .gitignore
        if (templates_dir / ".gitignore").exists():
            shutil.copy2(templates_dir / ".gitignore", backend_dir / ".gitignore")

        # Copy README (with templating)
        if (templates_dir / "README.md").exists():
            readme_template = (templates_dir / "README.md").read_text()
            readme_content = readme_template.replace("{{ project_name }}", context.project_name)
            readme_content = readme_content.replace("{{ project_slug }}", context.project_slug)
            readme_content = readme_content.replace("{{ description }}", context.description or "")
            write_file(backend_dir / "README.md", readme_content)

        # Copy tests directory
        tests_template_dir = templates_dir / "tests"
        if tests_template_dir.exists():
            tests_dir = backend_dir / "tests"
            ensure_dir(tests_dir)
            for test_file in tests_template_dir.glob("*.py"):
                shutil.copy2(test_file, tests_dir / test_file.name)

    @staticmethod
    def _create_pyproject_content(
        project_name: str,
        dependencies: list[str],
        dev_dependencies: list[str] | None = None,
    ) -> str:
        """Create pyproject.toml content.

        Args:
            project_name: Project name.
            dependencies: List of dependencies.
            dev_dependencies: Optional list of dev dependencies.

        Returns:
            Pyproject.toml content as string.

        """
        dev_dependencies = dev_dependencies or [
            "pytest>=8.0.0",
            "pytest-cov>=4.1.0",
            "pytest-asyncio>=0.23.0",
            "ruff>=0.4.0",
            "mypy>=1.9.0",
        ]

        deps_str = "\n".join(f'    "{dep}",' for dep in dependencies)
        dev_deps_str = "\n".join(f'    "{dep}",' for dep in dev_dependencies)

        return f"""[project]
name = "{project_name}-backend"
version = "0.1.0"
description = "Backend for {project_name}"
readme = "README.md"
requires-python = ">=3.11"
dependencies = [
{deps_str}
]

[project.optional-dependencies]
dev = [
{dev_deps_str}
]

[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"

[tool.uv]
dev-dependencies = [
{dev_deps_str}
]

[tool.ruff]
line-length = 120
target-version = "py311"

[tool.ruff.lint]
select = ["E", "F", "I", "UP", "B", "SIM"]
ignore = ["E501"]

[tool.pytest.ini_options]
asyncio_mode = "auto"
testpaths = ["tests"]
python_files = "test_*.py"
python_classes = "Test*"
python_functions = "test_*"

[tool.mypy]
python_version = "3.11"
strict = true
warn_return_any = true
warn_unused_configs = true
disallow_untyped_defs = true
"""



"""FastAPI backend generator."""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from litestar_start.backends.base import BackendGenerator
from litestar_start.core.templates import TemplateLoader
from litestar_start.core.utils import ensure_dir, write_file

if TYPE_CHECKING:
    from litestar_start.core.base import GeneratorContext


class FastAPIGenerator(BackendGenerator):
    """Generator for FastAPI backend projects."""

    def __init__(self) -> None:
        """Initialize the FastAPI generator."""
        super().__init__()
        self.template_loader = TemplateLoader()
        # Register FastAPI-specific templates
        self.template_loader.register_template_dir("fastapi", Path(__file__).parent / "templates")

    @property
    def name(self) -> str:
        """Generator name."""
        return "fastapi"

    @property
    def display_name(self) -> str:
        """Human-readable name."""
        return "FastAPI"

    def get_dependencies(self) -> list[str]:
        """Get FastAPI dependencies.

        Returns:
            List of package dependencies.

        """
        return [
            "fastapi>=0.109.0",
            "uvicorn[standard]>=0.27.0",
            "python-dotenv>=1.0.0",
            "msgspec>=0.20.0",
        ]

    def _generate_pyproject(self, backend_dir: Path, context: GeneratorContext) -> None:
        """Generate pyproject.toml for FastAPI.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        template_context = {
            "project_slug": context.project_slug,
            "description": context.description,
            "dependencies": self.get_dependencies(),
        }
        content = self.template_loader.render("fastapi:pyproject.toml.jinja", template_context)
        write_file(backend_dir / "pyproject.toml", content)

        # Create .python-version for uv
        write_file(backend_dir / ".python-version", "3.12\n")

        # Create .gitignore
        gitignore_content = self.template_loader.render("fastapi:gitignore.jinja", {})
        write_file(backend_dir / ".gitignore", gitignore_content)

    def _generate_app_structure(self, backend_dir: Path, context: GeneratorContext) -> None:
        """Generate FastAPI application structure.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        app_dir = backend_dir / "app"
        ensure_dir(app_dir)

        template_context = {
            "project_name": context.project_name,
            "project_slug": context.project_slug,
            "description": context.description,
        }

        # Create __init__.py
        init_content = self.template_loader.render("fastapi:app/__init__.py.jinja", {})
        write_file(app_dir / "__init__.py", init_content)

        # Create main.py
        main_content = self.template_loader.render("fastapi:app/main.py.jinja", template_context)
        write_file(app_dir / "main.py", main_content)

        # Create config.py
        config_content = self.template_loader.render("fastapi:app/config.py.jinja", template_context)
        write_file(app_dir / "config.py", config_content)

        # Create api directory
        api_dir = app_dir / "api"
        ensure_dir(api_dir)
        api_init_content = self.template_loader.render("fastapi:api/__init__.py.jinja", {})
        write_file(api_dir / "__init__.py", api_init_content)

        # Create routes.py
        routes_content = self.template_loader.render("fastapi:api/routes.py.jinja", {})
        write_file(api_dir / "routes.py", routes_content)



"""Generator registry for framework and plugin discovery."""

from __future__ import annotations

from collections import defaultdict
from typing import TYPE_CHECKING, ClassVar

if TYPE_CHECKING:
    from litestar_start.core.base import BaseGenerator, PluginInterface


class GeneratorRegistry:
    """Central registry for discovering generators and plugins."""

    _backends: ClassVar[dict[str, type[BaseGenerator]]] = {}
    _plugins: ClassVar[dict[str, dict[str, type[PluginInterface]]]] = defaultdict(dict)

    @classmethod
    def register_backend(cls, generator: type[BaseGenerator]) -> type[BaseGenerator]:
        """Register a backend generator. Can be used as decorator.

        Args:
            generator: Backend generator class to register.

        Returns:
            The same generator class (for decorator usage).

        """
        # Instantiate to get the name property
        instance = generator()
        cls._backends[instance.name] = generator
        return generator

    @classmethod
    def get_backend(cls, name: str) -> type[BaseGenerator] | None:
        """Get a backend generator by name.

        Args:
            name: Backend generator name.

        Returns:
            Backend generator class or None if not found.

        """
        return cls._backends.get(name)

    @classmethod
    def list_backends(cls) -> list[tuple[str, str]]:
        """List all registered backends with display names.

        Returns:
            List of (name, display_name) tuples for all backends.

        """
        return [(instance.name, instance.display_name) for instance in (gen() for gen in cls._backends.values())]

    @classmethod
    def register_plugin(cls, framework: str, plugin: type[PluginInterface]) -> type[PluginInterface]:
        """Register a plugin for a framework.

        Args:
            framework: Framework name (e.g., 'fastapi', 'litestar').
            plugin: Plugin class to register.

        Returns:
            The same plugin class (for decorator usage).

        """
        instance = plugin()
        cls._plugins[framework][instance.name] = plugin
        return plugin

    @classmethod
    def get_plugins_by_category(cls, framework: str, category: str) -> list[type[PluginInterface]]:
        """Get all plugins for a framework in a specific category.

        Args:
            framework: Framework name.
            category: Plugin category ('orm', 'auth', 'cache', etc.).

        Returns:
            List of plugin classes in the specified category.

        """
        return [plugin for plugin in cls._plugins.get(framework, {}).values() if plugin().category == category]

    @classmethod
    def get_plugins_for(cls, framework: str, category: str | None = None) -> list[type[PluginInterface]]:
        """Get all plugins for a framework, optionally filtered by category.

        Args:
            framework: Framework name.
            category: Optional category filter ('orm', 'auth', etc.).

        Returns:
            List of plugin classes matching the criteria.

        """
        framework_plugins = cls._plugins.get(framework, {})
        plugins = list(framework_plugins.values())

        if category:
            plugins = [p for p in plugins if p().category == category]

        return plugins

    @classmethod
    def list_plugins(cls, framework: str, category: str | None = None) -> list[tuple[str, str]]:
        """List plugins for a framework with display names.

        Args:
            framework: Framework name.
            category: Optional category filter.

        Returns:
            List of (name, display_name) tuples for plugins.

        """
        plugins = cls.get_plugins_for(framework, category)
        return [(p().name, p().display_name) for p in plugins]

    @classmethod
    def get_plugin(cls, framework: str, plugin_name: str) -> type[PluginInterface] | None:
        """Get a specific plugin for a framework.

        Args:
            framework: Framework name.
            plugin_name: Plugin name.

        Returns:
            Plugin class or None if not found.

        """
        return cls._plugins.get(framework, {}).get(plugin_name)

    @classmethod
    def clear(cls) -> None:
        """Clear all registrations. Primarily for testing."""
        cls._backends.clear()
        cls._plugins.clear()



"""Main project orchestrator for generating complete projects."""

from __future__ import annotations

from typing import TYPE_CHECKING

from rich.console import Console

from litestar_start.core.base import GeneratorContext
from litestar_start.core.utils import ensure_dir, write_file
from litestar_start.generators.registry import GeneratorRegistry

if TYPE_CHECKING:
    from litestar_start.core.config import ProjectConfig

console = Console()


class ProjectOrchestrator:
    """Orchestrates the generation of a complete project."""

    def __init__(self, config: ProjectConfig) -> None:
        """Initialize the project orchestrator.

        Args:
            config: Project configuration.

        """
        self.config = config
        self.output_dir = config.output_dir.resolve()
        ensure_dir(self.output_dir)

    def generate(self) -> None:
        """Generate the complete project."""
        context = self._create_context()

        # Generate backend
        self._generate_backend(context)

        # Generate root files
        self._generate_root_files()

        # Generate docker-compose if docker enabled
        if self.config.has_docker:
            self._generate_docker_compose()

        console.print(f"[dim]Generated project at {self.output_dir}[/dim]")

    def _create_context(self) -> GeneratorContext:
        """Create generator context from config.

        Returns:
            GeneratorContext instance.

        """
        return GeneratorContext(
            project_name=self.config.project_name,
            project_slug=self.config.project_slug,
            description=self.config.description,
            output_dir=self.output_dir,
            config=self.config.to_context(),
        )

    def _generate_backend(self, context: GeneratorContext) -> None:
        """Generate backend code."""
        backend_gen_class = GeneratorRegistry.get_backend(self.config.backend)
        if backend_gen_class:
            backend_gen = backend_gen_class()
            backend_gen.generate(context)
        else:
            console.print(f"[red]Error: No generator found for backend '{self.config.backend}'[/red]")

    def _generate_root_files(self) -> None:
        """Generate root project files."""
        self._generate_gitignore()
        self._generate_env_example()
        self._generate_readme()

    def _generate_gitignore(self) -> None:
        """Generate .gitignore file."""
        gitignore_content = """\
# Python
__pycache__/
*.py[cod]
*$py.class
*.so
.Python
venv/
.venv/
ENV/
.env
uv.lock

# Node
node_modules/
dist/
.cache/

# IDE
.idea/
.vscode/
*.swp
*.swo

# OS
.DS_Store
Thumbs.db

# Project
*.log
.coverage
htmlcov/
"""
        write_file(self.output_dir / ".gitignore", gitignore_content)

    def _generate_env_example(self) -> None:
        """Generate .env.example file."""
        env_lines = [
            f"# {self.config.project_name} Environment Variables",
            "",
            "# Application",
            f"APP_NAME={self.config.project_slug}",
            "APP_ENV=development",
            "DEBUG=true",
            "",
            "# Database",
        ]

        db_urls = {
            "postgresql": f"DATABASE_URL=postgresql://user:password@localhost:5432/{self.config.project_slug}",
            "mysql": f"DATABASE_URL=mysql://user:password@localhost:3306/{self.config.project_slug}",
            "sqlite": "DATABASE_URL=sqlite:///./app.db",
            "mongodb": f"MONGODB_URL=mongodb://localhost:27017/{self.config.project_slug}",
        }

        if self.config.database in db_urls:
            env_lines.append(db_urls[self.config.database])

        env_lines.extend(["", "# Authentication"])
        if self.config.has_auth:
            env_lines.append("SECRET_KEY=your-secret-key-here")
        if self.config.auth == "jwt":
            env_lines.append("JWT_SECRET=your-jwt-secret-here")

        write_file(self.output_dir / ".env.example", "\n".join(env_lines) + "\n")

    def _generate_readme(self) -> None:
        """Generate README.md file."""
        auth_text = self.config.auth.title() if self.config.has_auth else "None"

        readme_lines = [
            f"# {self.config.project_name}",
            "",
            self.config.description,
            "",
            "## Tech Stack",
            "",
            f"- **Framework**: {self.config.backend.title()}",
            f"- **Database**: {self.config.database.title()}",
            f"- **ORM**: {self.config.orm.title()}",
            f"- **Auth**: {auth_text}",
            "",
            "## Getting Started",
            "",
            "```bash",
            "uv sync",
            "uv run uvicorn app.main:app --reload",
            "```",
        ]

        readme_lines.extend(["", "## License", "", "MIT"])

        write_file(self.output_dir / "README.md", "\n".join(readme_lines) + "\n")

    def _generate_docker_compose(self) -> None:
        """Generate docker-compose.yml file."""
        compose_lines = [
            "services:",
            "  app:",
            "    build:",
            "      context: .",
            "      dockerfile: Dockerfile",
            "    ports:",
            '      - "8000:8000"',
            "    environment:",
            "      - APP_ENV=development",
            f"      - APP_NAME={self.config.project_slug}",
        ]

        if self.config.database not in {"sqlite", "none"}:
            compose_lines.extend(
                [
                    "    depends_on:",
                    "      - db",
                ],
            )

        compose_lines.extend(
            [
                "    volumes:",
                "      - .:/app",
            ],
        )

        if self.config.database == "postgresql":
            compose_lines.extend(
                [
                    "",
                    "  db:",
                    "    image: postgres:16-alpine",
                    "    environment:",
                    "      POSTGRES_USER: postgres",
                    "      POSTGRES_PASSWORD: postgres",
                    f"      POSTGRES_DB: {self.config.project_slug}",
                    "    ports:",
                    '      - "5432:5432"',
                    "    volumes:",
                    "      - postgres_data:/var/lib/postgresql/data",
                    "",
                    "volumes:",
                    "  postgres_data:",
                ],
            )
        elif self.config.database == "mongodb":
            compose_lines.extend(
                [
                    "",
                    "  db:",
                    "    image: mongo:7",
                    "    ports:",
                    '      - "27017:27017"',
                    "    volumes:",
                    "      - mongo_data:/data/db",
                    "",
                    "volumes:",
                    "  mongo_data:",
                ],
            )

        write_file(self.output_dir / "docker-compose.yml", "\n".join(compose_lines) + "\n")



"""Litestar Start - Interactive CLI for scaffolding fullstack projects."""

from litestar_start.core.config import ProjectConfig
from litestar_start.generators.loader import register_all_generators
from litestar_start.generators.project import ProjectOrchestrator
from litestar_start.generators.registry import GeneratorRegistry

__all__ = [
    "GeneratorRegistry",
    "ProjectConfig",
    "ProjectOrchestrator",
    "register_all_generators",
]



"""Configuration models for project generation."""

from __future__ import annotations

from pathlib import Path
from typing import Any

import msgspec


class ProjectConfig(msgspec.Struct):
    """Project configuration model."""

    project_name: str
    project_slug: str
    description: str
    backend: str
    database: str
    orm: str
    auth: str
    features: list[str] = []
    output_dir: Path = Path()

    @property
    def has_docker(self) -> bool:
        """Check if Docker is enabled."""
        return "docker" in self.features

    @property
    def has_cicd(self) -> bool:
        """Check if CI/CD is enabled."""
        return "cicd" in self.features

    @property
    def has_testing(self) -> bool:
        """Check if testing is enabled."""
        return "testing" in self.features

    @property
    def has_linting(self) -> bool:
        """Check if linting is enabled."""
        return "linting" in self.features

    @property
    def has_auth(self) -> bool:
        """Check if authentication is enabled."""
        return self.auth != "none"

    def to_context(self) -> dict[str, Any]:
        """Convert to template context dictionary.

        Returns:
            Dictionary suitable for template rendering.

        """
        return {
            "project_name": self.project_name,
            "project_slug": self.project_slug,
            "description": self.description,
            "backend": self.backend,
            "database": self.database,
            "orm": self.orm,
            "auth": self.auth,
            "features": self.features,
            "docker": self.has_docker,
            "cicd": self.has_cicd,
            "testing": self.has_testing,
            "linting": self.has_linting,
        }

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> ProjectConfig:
        """Create ProjectConfig from dictionary.

        Args:
            data: Configuration dictionary.

        Returns:
            ProjectConfig instance.

        """
        output_dir = data.get("output_dir", ".")
        if isinstance(output_dir, str):
            output_dir = Path(output_dir)

        return cls(
            project_name=data["project_name"],
            project_slug=data.get("project_slug", data["project_name"].lower().replace(" ", "-")),
            description=data.get("description", ""),
            backend=data["backend"],
            database=data.get("database", "sqlite"),
            orm=data.get("orm", "none"),
            auth=data.get("auth", "none"),
            features=data.get("features", []),
            output_dir=output_dir,
        )



"""Register all generators with the registry."""

from litestar_start.backends.fastapi import FastAPIGenerator
from litestar_start.backends.litestar import LitestarGenerator
from litestar_start.generators.registry import GeneratorRegistry


def register_all_generators() -> None:
    """Register all available generators."""
    # Register backends
    GeneratorRegistry.register_backend(LitestarGenerator)
    GeneratorRegistry.register_backend(FastAPIGenerator)



"""Template loading and rendering system."""

from __future__ import annotations

import datetime
from pathlib import Path
from typing import TYPE_CHECKING, Any, ClassVar

from jinja2 import Environment, FileSystemLoader, PrefixLoader

if TYPE_CHECKING:
    from jinja2 import Template


class TemplateRenderError(Exception):
    """Raised when template rendering fails."""

    def __init__(self, template_name: str, original_error: Exception, context: dict[str, Any]) -> None:
        """Initialize error with context.

        Args:
            template_name: Name of the template that failed.
            original_error: The original exception.
            context: The context that was provided.

        """
        self.template_name = template_name
        self.original_error = original_error
        self.context = context

        message = f"""
Template rendering failed: {template_name}

Error: {original_error}

Available context keys: {list(context.keys())}

Tip: Check that all required variables are provided in the context.
"""
        super().__init__(message)


class TemplateValidator:
    """Validates template context before rendering."""

    REQUIRED_CONTEXT: ClassVar[dict[str, list[str]]] = {
        "base:gitignore.jinja": [],
        "fastapi:app/main.py.jinja": ["project_name", "description"],
        "fastapi:pyproject.toml.jinja": ["project_slug", "dependencies"],
        "litestar:app/main.py.jinja": ["project_name", "description"],
        "litestar:pyproject.toml.jinja": ["project_slug", "dependencies"],
    }

    @classmethod
    def validate(cls, template_name: str, context: dict[str, Any]) -> list[str]:
        """Validate context has required keys. Returns list of errors.

        Args:
            template_name: Name of the template to validate.
            context: Context dictionary to validate.

        Returns:
            List of validation error messages.

        """
        required = cls.REQUIRED_CONTEXT.get(template_name, [])
        return [f"Missing required context key: {key}" for key in required if key not in context]


class TemplateLoader:
    """Discovers and loads templates from multiple locations."""

    def __init__(self) -> None:
        """Initialize the template loader."""
        self._base_path = Path(__file__).parent / "templates" / "base"
        self._env: Environment | None = None
        self._loaders: dict[str, FileSystemLoader] = {}

    def register_template_dir(self, prefix: str, path: Path) -> None:
        """Register a template directory with a prefix.

        Args:
            prefix: Prefix for template names (e.g., "fastapi", "litestar").
            path: Path to the template directory.

        """
        if path.exists():
            self._loaders[prefix] = FileSystemLoader(str(path))
            self._env = None  # Reset environment to pick up new loader

    def _build_environment(self) -> Environment:
        """Build Jinja environment with all registered loaders.

        Returns:
            Configured Jinja2 Environment.

        """
        loaders: dict[str, FileSystemLoader] = {
            "base": FileSystemLoader(str(self._base_path)),
            **self._loaders,
        }

        env = Environment(
            loader=PrefixLoader(loaders, delimiter=":"),
            autoescape=False,  # We're generating code, not HTML
            keep_trailing_newline=True,
            trim_blocks=True,
            lstrip_blocks=True,
        )

        # Add custom filters and globals
        env.filters.update(self._get_custom_filters())
        env.globals.update(self._get_custom_globals())

        return env

    @property
    def env(self) -> Environment:
        """Get or create the Jinja environment.

        Returns:
            The Jinja2 Environment instance.

        """
        if self._env is None:
            self._env = self._build_environment()
        return self._env

    def render(self, template_name: str, context: dict[str, Any]) -> str:
        """Render a template with the given context.

        Args:
            template_name: Name of the template (e.g., "fastapi:app/main.py.jinja").
            context: Context dictionary for template rendering.

        Returns:
            Rendered template string.

        Raises:
            ValueError: If template context validation fails.
            TemplateRenderError: If template rendering fails.

        """
        # Validate context
        validation_errors = TemplateValidator.validate(template_name, context)
        if validation_errors:
            msg = f"Template context validation failed: {', '.join(validation_errors)}"
            raise ValueError(msg)

        try:
            template = self.env.get_template(template_name)
            return template.render(**context)
        except Exception as e:
            raise TemplateRenderError(template_name, e, context) from e

    @staticmethod
    def _get_custom_filters() -> dict[str, Any]:
        """Get custom Jinja filters for code generation.

        Returns:
            Dictionary of custom filters.

        """
        return {
            "snake_case": lambda s: s.lower().replace("-", "_").replace(" ", "_"),
            "pascal_case": lambda s: "".join(w.title() for w in s.replace("-", "_").split("_")),
            "kebab_case": lambda s: s.lower().replace("_", "-").replace(" ", "-"),
            "quote": lambda s: f'"{s}"',
            "indent": lambda s, n: "\n".join(" " * n + line for line in s.split("\n")),
        }

    @staticmethod
    def _get_custom_globals() -> dict[str, Any]:
        """Get custom Jinja globals.

        Returns:
            Dictionary of custom globals.

        """
        return {
            "now": lambda: datetime.datetime.now(tz=datetime.timezone.utc),
            "year": lambda: datetime.datetime.now(tz=datetime.timezone.utc).year,
        }


class CachedTemplateLoader(TemplateLoader):
    """Template loader with caching for production."""

    def __init__(self, cache_size: int = 128) -> None:
        """Initialize cached template loader.

        Args:
            cache_size: Maximum number of compiled templates to cache.

        """
        super().__init__()
        self._cache_size = cache_size
        self._template_cache: dict[str, Template] = {}

    def _get_compiled_template(self, template_name: str) -> Template:
        """Get compiled template from cache or compile it.

        Args:
            template_name: Name of the template.

        Returns:
            Compiled Jinja2 template.

        """
        if template_name not in self._template_cache:
            self._template_cache[template_name] = self.env.get_template(template_name)
        return self._template_cache[template_name]

    def render(self, template_name: str, context: dict[str, Any]) -> str:
        """Render a template with caching.

        Args:
            template_name: Name of the template.
            context: Context dictionary for rendering.

        Returns:
            Rendered template string.

        Raises:
            ValueError: If template context validation fails.
            TemplateRenderError: If template rendering fails.

        """
        # Validate context
        validation_errors = TemplateValidator.validate(template_name, context)
        if validation_errors:
            msg = f"Template context validation failed: {', '.join(validation_errors)}"
            raise ValueError(msg)

        try:
            template = self._get_compiled_template(template_name)
            return template.render(**context)
        except Exception as e:
            raise TemplateRenderError(template_name, e, context) from e

    def clear_cache(self) -> None:
        """Clear the template cache."""
        self._template_cache.clear()



"""Abstract base classes for generators and plugins."""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any

import msgspec

if TYPE_CHECKING:
    from pathlib import Path


class GeneratorContext(msgspec.Struct):
    """Context passed to all generators."""

    project_name: str
    project_slug: str
    description: str
    output_dir: Path
    config: dict[str, Any]


class BaseGenerator(ABC):
    """Abstract base for all generators."""

    @property
    @abstractmethod
    def name(self) -> str:
        """Unique identifier for this generator."""
        ...

    @property
    @abstractmethod
    def display_name(self) -> str:
        """Human-readable name for CLI display."""
        ...

    @abstractmethod
    def generate(self, context: GeneratorContext) -> None:
        """Generate files for this component.

        Args:
            context: Generator context with project configuration.

        """
        ...

    @abstractmethod
    def get_dependencies(self) -> list[str]:
        """Return list of package dependencies.

        Returns:
            List of package names with version specifiers.

        """
        ...


class PluginInterface(ABC):
    """Interface for framework plugins (ORMs, auth, etc.)."""

    @property
    @abstractmethod
    def name(self) -> str:
        """Plugin identifier."""
        ...

    @property
    @abstractmethod
    def display_name(self) -> str:
        """Human-readable name for CLI display."""
        ...

    @property
    @abstractmethod
    def category(self) -> str:
        """Plugin category: 'orm', 'auth', 'cache', etc."""
        ...

    @abstractmethod
    def apply(self, context: GeneratorContext, base_files: dict[str, str]) -> dict[str, str]:
        """Modify or add files.

        Args:
            context: Generator context with project configuration.
            base_files: Dictionary mapping file paths to content.

        Returns:
            Updated file mapping with plugin modifications.

        """
        ...

    @abstractmethod
    def get_dependencies(self) -> list[str]:
        """Additional dependencies this plugin requires.

        Returns:
            List of package names with version specifiers.

        """
        ...

    @staticmethod
    def get_env_vars() -> dict[str, str]:
        """Environment variables to add to .env.example.

        Returns:
            Dictionary mapping variable names to default values.

        """
        return {}



"""Shared utility functions."""

from __future__ import annotations

from pathlib import Path


def ensure_dir(path: Path) -> Path:
    """Ensure a directory exists, creating it if necessary.

    Args:
        path: Directory path to ensure exists.

    Returns:
        The directory path.

    """
    path.mkdir(parents=True, exist_ok=True)
    return path


def slugify(text: str) -> str:
    """Convert text to a slug-friendly format.

    Args:
        text: Text to slugify.

    Returns:
        Slugified text.

    """
    return text.lower().replace(" ", "-").replace("_", "-")


def write_file(path: Path, content: str) -> None:
    """Write content to a file, creating parent directories if needed.

    Args:
        path: File path to write to.
        content: Content to write.

    """
    ensure_dir(path.parent)
    path.write_text(content, encoding="utf-8")



"""Litestar backend generator."""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from litestar_start.backends.base import BackendGenerator
from litestar_start.core.templates import TemplateLoader
from litestar_start.core.utils import ensure_dir, write_file

if TYPE_CHECKING:
    from litestar_start.core.base import GeneratorContext


class LitestarGenerator(BackendGenerator):
    """Generator for Litestar backend projects."""

    def __init__(self) -> None:
        """Initialize the Litestar generator."""
        super().__init__()
        self.template_loader = TemplateLoader()
        # Register Litestar-specific templates
        self.template_loader.register_template_dir("litestar", Path(__file__).parent / "templates")

    @property
    def name(self) -> str:
        """Generator name."""
        return "litestar"

    @property
    def display_name(self) -> str:
        """Human-readable name."""
        return "Litestar"

    def get_dependencies(self) -> list[str]:
        """Get Litestar dependencies.

        Returns:
            List of package dependencies.

        """
        return [
            "litestar[standard]>=2.19.0",
            "python-dotenv>=1.2.1",
        ]

    def _generate_pyproject(self, backend_dir: Path, context: GeneratorContext) -> None:
        """Generate pyproject.toml for Litestar.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        template_context = {
            "project_slug": context.project_slug,
            "description": context.description,
            "dependencies": self.get_dependencies(),
        }
        content = self.template_loader.render("litestar:pyproject.toml.jinja", template_context)
        write_file(backend_dir / "pyproject.toml", content)

        # Create .python-version for uv
        write_file(backend_dir / ".python-version", "3.13\n")

        # Create .gitignore
        gitignore_content = self.template_loader.render("litestar:gitignore.jinja", {})
        write_file(backend_dir / ".gitignore", gitignore_content)

    def _generate_app_structure(self, backend_dir: Path, context: GeneratorContext) -> None:
        """Generate Litestar application structure.

        Args:
            backend_dir: Backend directory path.
            context: Generator context.

        """
        app_dir = backend_dir / "app"
        ensure_dir(app_dir)

        template_context = {
            "project_name": context.project_name,
            "project_slug": context.project_slug,
            "description": context.description,
        }

        # Create __init__.py
        init_content = self.template_loader.render("litestar:app/__init__.py.jinja", {})
        write_file(app_dir / "__init__.py", init_content)

        # Create main.py
        main_content = self.template_loader.render("litestar:app/main.py.jinja", template_context)
        write_file(app_dir / "main.py", main_content)

        # Create config.py
        config_content = self.template_loader.render("litestar:app/config.py.jinja", template_context)
        write_file(app_dir / "config.py", config_content)

        # Create controllers directory
        controllers_dir = app_dir / "controllers"
        ensure_dir(controllers_dir)
        controllers_init_content = self.template_loader.render("litestar:controllers/__init__.py.jinja", {})
        write_file(controllers_dir / "__init__.py", controllers_init_content)

        # Create base controller
        controller_content = self.template_loader.render("litestar:controllers/base.py.jinja", {})
        write_file(controllers_dir / "base.py", controller_content)



"""Core abstractions for the litestar-start generator."""

from litestar_start.core.base import BaseGenerator, GeneratorContext
from litestar_start.core.config import ProjectConfig

__all__ = [
    "BaseGenerator",
    "GeneratorContext",
    "ProjectConfig",
]
